#include "enlightenment.h"
void hangul_font_init(void);

void FontField(char *font, int num, char *ret)
{
   int count;
   int i,j,k,l;
   
   i=0;
   count=-1;
   while (count<num)
     {
	if (font[i++]=='-') count++;
     }
   j=i;
   while (font[j++]!='-');
   l=0;
   for (k=i;k<j-1;k++)
     ret[l++]=font[k];
   ret[l]=0;
}

void InitTextInfo()
{
   char **fonts;
   char pat[1024];
   int num;
   int i;
   int flag;
   char field[1024];
   
   sprintf(pat,"-*-%s-*-*-*-*-*-*-*-*-*-*-*-*",cfg.font);
   fonts=XListFonts(disp,pat,10240,&num);
   flag=0;
	for (i=0;i<num;i++)
     {
	FontField(fonts[i],1,field);
	if (!strcmp(cfg.font,field)) flag=1;
     }
   if (!flag)
     {
	free(cfg.font);
	cfg.font=malloc(6);
	strcpy(cfg.font,"fixed");
     }
   if (cfg.font_style&BOLD)
     {
	flag=0;
	for (i=0;i<num;i++)
	  {
	     FontField(fonts[i],2,field);
	     if (!strcmp("bold",field)) flag=1;
	  }
	if (flag) strcpy(font_weight,"bold");
	else strcpy(font_weight,"*");
     }
   else 
     {
	flag=0;
	for (i=0;i<num;i++)
	  {
	     FontField(fonts[i],2,field);
	     if (!strcmp("medium",field)) flag=1;
			 }
	if (flag) strcpy(font_weight,"medium");
	else strcpy(font_weight,"*");
     }
   if (cfg.font_style&ITALIC)
     {
	flag=0;
		  for (i=0;i<num;i++)
	  {
	     FontField(fonts[i],3,field);
	     if (!strcmp("i",field)) flag=1;
	  }
	if (flag) strcpy(font_slant,"i");
	else strcpy (font_slant,"*");
     }
   else 
     {
	flag=0;
	for (i=0;i<num;i++)
	  {
	     FontField(fonts[i],3,field);
	     if (!strcmp("r",field)) flag=1;
			 }
	if (flag) strcpy(font_slant,"r");
	else strcpy(font_slant,"*");
     }
   XFreeFontNames(fonts);					  
   hangul_font_init();
}

static char **hangul_font_list;
static int hangul_font_number;

void
hangul_font_init(void)
{
  char **fonts;
  char pat[256];
  int num;
  if (cfg.hfont_style & BOLD) {
    sprintf(pat,"-*-%s-bold-*-ksc5601.1987-*", cfg.hfont);
  } else {
    sprintf(pat,"-*-%s-medium-*-ksc5601.1987-*",cfg.hfont);
  }
  fonts=XListFonts(disp,pat,10240,&num);
  if (num == 0) {
    XFreeFontNames(fonts);
    sprintf(pat,"-*-%s-*-ksc5601.1987-0",cfg.hfont);
    fonts = XListFonts(disp, pat, 1024, &num);
    if (num == 0) {
      XFreeFontNames(fonts);
      sprintf(pat,"*-ksc5601.1987-0");
      fonts = XListFonts(disp, pat, 1024, &num);
    }
  }
  hangul_font_list = fonts;
  hangul_font_number = num;
}

int
hangul_font_height(char *fontname)
{
  char height[4];
  FontField(fontname, 11, height);
  return atoi(height)/10;
}

char *
get_hangul_fontname(int height)
{
  int i;
  do {
    for (i = 0; i < hangul_font_number; i++) {
      if (hangul_font_height(hangul_font_list[i]) == height)
        return hangul_font_list[i];
    }
  } while (height-- > 4);
  return NULL;
}

void DrawText(Pixmap p, char *txt, int px, int py, int w, int h)
{
   GC fg;
   GC bg;
   XGCValues gcv;
   int hh;
   int i;
   int x,y,ww;
   XFontSet fs;
   XFontStruct *xfs;
   XCharStruct dummy2;
   Font font;
   int dummy;
   int ascent;
   int descent;
   char font_string[256];
   char fontname[256];
   char tstr[32];
   char *hangul_fontname;
   char **missing_set, *def_string;
   int missing_count;
   int mode;
   
   hh=h-2;
   if (hh<2) return;
   for (i=hh;i>2;i--)
     {
	sprintf(font_string,"-*-%s-%s-%s-*-*-%i-*-*-*-*-*-*-*",
		cfg.font,font_weight,font_slant,i);
	font=XLoadFont(disp,font_string);
	if (!font) return;
	xfs=XLoadQueryFont(disp,font_string);
	if (!xfs) return;
	XTextExtents(xfs,txt,strlen(txt),&dummy,&ascent,&descent,&dummy2);
	if ((ascent+descent)<=hh) break;
	XFreeFont(disp,xfs);
     }
   XFreeFont(disp, xfs);
   hangul_fontname = get_hangul_fontname(hh);
   sprintf(fontname, "%s,%s", font_string, hangul_fontname);
   fs = XCreateFontSet(disp, fontname, &missing_set, &missing_count, &def_string);
   
   fg=XCreateGC(disp,p,0,&gcv);
   bg=XCreateGC(disp,p,0,&gcv);
#if 0
   XSetFont(disp,fg,font);
   XSetFont(disp,bg,font);
#endif
   XSetForeground(disp,fg,cfg.font_fg);
   XSetForeground(disp,bg,cfg.font_bg);
   
   if (cfg.font_style&OUTLINE) mode=1;
   else if (cfg.font_style&SHADOW) mode=2;
   else mode=0;
   
   for (x=strlen(txt);x>0;x--)
     {
	i=XmbTextEscapement(fs,txt,x);
	if (i<(w-2))
	  {
	     y=1+ascent;
	     ww=x;
	     if (cfg.font_style&J_RIGHT) x=w-i-1;
	     else if (cfg.font_style&J_CENTER) x=(w>>1)-(i>>1);
	     else x=1;
	     if (mode==0)
	       {
		  XmbDrawString(disp,p,fs,fg,px+x,py+y,txt,ww);
	       }
	     else if (mode==1)
	       {
		  XmbDrawString(disp,p,fs,bg,px+x-1,py+y,txt,ww);
		  XmbDrawString(disp,p,fs,bg,px+x+1,py+y,txt,ww);
		  XmbDrawString(disp,p,fs,bg,px+x,py+y-1,txt,ww);
		  XmbDrawString(disp,p,fs,bg,px+x,py+y+1,txt,ww);
		  XmbDrawString(disp,p,fs,fg,px+x,py+y,txt,ww);
	       }
	     else if (mode==2)
	       {
		  XmbDrawString(disp,p,fs,bg,px+x+1,py+y+1,txt,ww);
		  XmbDrawString(disp,p,fs,fg,px+x,py+y,txt,ww);
	       }
	     x=-1;
	     break;
	  }
	if (txt[x - 1] & 0x80) x--;
     }
   XFreeFontSet(disp,fs);
   XFreeGC(disp,fg);
   XFreeGC(disp,bg);
}

void DrawTextMask(Pixmap p, char *txt, int px, int py, int w, int h)
{
   GC or;
   XGCValues gcv;
   int hh;
   int i;
   int x,y,ww;
   XFontSet fs;
   XFontStruct *xfs;
   XCharStruct dummy2;
   Font font;
   int dummy;
   int ascent;
   int descent;
   char font_string[256];
   char fontname[256];
   char *hangul_fontname;
   char **missing_set, *def_string;
   int missing_count;
   char tstr[32];
   int mode;
   
   hh=h-2;
   if (hh<2) return;
   for (i=hh;i>2;i--)
     {
	sprintf(font_string,"-*-%s-%s-%s-*-*-%i-*-*-*-*-*-*-*",
		cfg.font,font_weight,font_slant,i);
	font=XLoadFont(disp,font_string);
	if (!font) return;
	xfs=XLoadQueryFont(disp,font_string);
	if (!xfs) return;
	XTextExtents(xfs,txt,strlen(txt),&dummy,&ascent,&descent,&dummy2);
	if ((ascent+descent)<=hh) break;
	XFreeFont(disp,xfs);
     }
   XFreeFont(disp, xfs);
   hangul_fontname = get_hangul_fontname(hh);
   sprintf(fontname, "%s,%s", font_string, hangul_fontname);
   fs = XCreateFontSet(disp, fontname, &missing_set, &missing_count, &def_string);
   gcv.function=GXor;
   or=XCreateGC(disp,p,GCFunction,&gcv);
   XSetForeground(disp,or,1);
   
   if (cfg.font_style&OUTLINE) mode=1;
   else if (cfg.font_style&SHADOW) mode=2;
   else mode=0;
   
   for (x=strlen(txt);x>0;x--)
     {
	i=XmbTextEscapement(fs,txt,x);
	if (i<(w-2))
	  {
	     y=1+ascent;
	     ww=x;
	     if (cfg.font_style&J_RIGHT) x=w-i-1;
	     else if (cfg.font_style&J_CENTER) x=(w>>1)-(i>>1);
	     else x=1;
	     if (mode==0)
	       {
		  XmbDrawString(disp,p,fs,or,px+x,py+y,txt,ww);
	       }
	     else if (mode==1)
	       {
		  XmbDrawString(disp,p,fs,or,px+x-1,py+y,txt,ww);
		  XmbDrawString(disp,p,fs,or,px+x+1,py+y,txt,ww);
		  XmbDrawString(disp,p,fs,or,px+x,py+y-1,txt,ww);
		  XmbDrawString(disp,p,fs,or,px+x,py+y+1,txt,ww);
		  XmbDrawString(disp,p,fs,or,px+x,py+y,txt,ww);
	       }
	     else if (mode==2)
	       {
		  XmbDrawString(disp,p,fs,or,px+x+1,py+y+1,txt,ww);
		  XmbDrawString(disp,p,fs,or,px+x,py+y,txt,ww);
	       }
	     x=-1;
	     break;
	  }
	if (txt[x - 1] & 0x80) x--;
     }
   XFreeGC(disp,or);
}
